from highlight_io.integrations import Integration


class VertexAIIntegration(Integration):
    INTEGRATION_KEY = "vertexai"

    def instrumentor(self):
        from opentelemetry.instrumentation.vertexai import VertexAIInstrumentor

        return VertexAIInstrumentor()



from highlight_io.integrations import Integration


class OpenAIIntegration(Integration):
    INTEGRATION_KEY = "openai"

    def instrumentor(self):
        from opentelemetry.instrumentation.openai import OpenAIInstrumentor

        return OpenAIInstrumentor()



from highlight_io.integrations import Integration


class ChromadbIntegration(Integration):
    INTEGRATION_KEY = "chromadb"

    def instrumentor(self):
        from opentelemetry.instrumentation.chromadb import ChromaInstrumentor

        return ChromaInstrumentor()



from highlight_io.integrations import Integration


class LlamaIndexIntegration(Integration):
    INTEGRATION_KEY = "llamaindex"

    def instrumentor(self):
        from opentelemetry.instrumentation.llamaindex import LlamaIndexInstrumentor

        return LlamaIndexInstrumentor()



from highlight_io.integrations import Integration


class BedrockIntegration(Integration):
    INTEGRATION_KEY = "bedrock"

    def instrumentor(self):
        from opentelemetry.instrumentation.bedrock import BedrockInstrumentor

        return BedrockInstrumentor()



from highlight_io.integrations import Integration


class AnthropicIntegration(Integration):
    INTEGRATION_KEY = "anthropic"

    def instrumentor(self):
        from opentelemetry.instrumentation.anthropic import AnthropicInstrumentor

        return AnthropicInstrumentor()



from highlight_io.integrations import Integration


class CeleryIntegration(Integration):
    INTEGRATION_KEY = "celery"

    def instrumentor(self):
        from opentelemetry.instrumentation.celery import CeleryInstrumentor

        return CeleryInstrumentor()



from highlight_io.integrations import Integration


class RequestsIntegration(Integration):
    INTEGRATION_KEY = "requests"

    def instrumentor(self):
        from opentelemetry.instrumentation.requests import RequestsInstrumentor

        return RequestsInstrumentor()



import functools

from highlight_io import H


def observe_serverless(get_highlight_header, fn):
    """
    Generic decorator for serverless request handlers. Extracts the request context
    to associate the request in azure functions, aws lambda, gcp, etc.
    :param highlight_header_value: the highlight header value from the current serverless request.
    :param fn: a function handler that the decorator is applied to.
    :return: a wrapped function that will record exceptions.
    """

    @functools.wraps(fn)
    def wrapper(*args, **kwargs):
        session_id, request_id = "", ""
        try:
            session_id, request_id = get_highlight_header(*args, **kwargs).split("/")
        except ValueError:
            pass
        try:
            with H.get_instance().trace("observe_serverless", session_id, request_id):
                return fn(*args, **kwargs)
        finally:
            # cloud functions may terminate quickly after response is sent.
            # flush to make sure logs / traces are delivered.
            H.get_instance().flush()

    return wrapper



from highlight_io.integrations import Integration


class HaystackIntegration(Integration):
    INTEGRATION_KEY = "haystack"

    def instrumentor(self):
        from opentelemetry.instrumentation.haystack import HaystackInstrumentor

        return HaystackInstrumentor()



from highlight_io.integrations import Integration


class BotoIntegration(Integration):
    INTEGRATION_KEY = "boto"

    def instrumentor(self):
        from opentelemetry.instrumentation.boto import BotoInstrumentor

        return BotoInstrumentor()



from highlight_io.integrations.anthropic import AnthropicIntegration
from highlight_io.integrations.bedrock import BedrockIntegration
from highlight_io.integrations.boto import BotoIntegration
from highlight_io.integrations.boto3sqs import Boto3SQSIntegration
from highlight_io.integrations.celery import CeleryIntegration
from highlight_io.integrations.chromadb import ChromadbIntegration
from highlight_io.integrations.cohere import CohereIntegration
from highlight_io.integrations.haystack import HaystackIntegration
from highlight_io.integrations.langchain import LangchainIntegration
from highlight_io.integrations.llamaindex import LlamaIndexIntegration
from highlight_io.integrations.openai import OpenAIIntegration
from highlight_io.integrations.pinecone import PineconeIntegration
from highlight_io.integrations.qdrant import QdrantInstrumentation
from highlight_io.integrations.redis import RedisIntegration
from highlight_io.integrations.replicate import ReplicateIntegration
from highlight_io.integrations.requests import RequestsIntegration
from highlight_io.integrations.sqlalchemy import SQLAlchemyIntegration
from highlight_io.integrations.transformers import TransformersIntegration
from highlight_io.integrations.vertexai import VertexAIIntegration
from highlight_io.integrations.watsonx import WatsonXIntegration
from highlight_io.integrations.weaviate import WeaviateIntegration

DEFAULT_INTEGRATIONS = [
    AnthropicIntegration,
    BedrockIntegration,
    Boto3SQSIntegration,
    BotoIntegration,
    CeleryIntegration,
    ChromadbIntegration,
    CohereIntegration,
    HaystackIntegration,
    LangchainIntegration,
    LlamaIndexIntegration,
    OpenAIIntegration,
    PineconeIntegration,
    QdrantInstrumentation,
    RedisIntegration,
    ReplicateIntegration,
    RequestsIntegration,
    SQLAlchemyIntegration,
    TransformersIntegration,
    VertexAIIntegration,
    WatsonXIntegration,
    WeaviateIntegration,
]



from highlight_io.integrations import Integration


class RedisIntegration(Integration):
    INTEGRATION_KEY = "redis"

    def instrumentor(self):
        from opentelemetry.instrumentation.redis import RedisInstrumentor

        return RedisInstrumentor()



from highlight_io.integrations import Integration


class ReplicateIntegration(Integration):
    INTEGRATION_KEY = "replicate"

    def instrumentor(self):
        from opentelemetry.instrumentation.replicate import ReplicateInstrumentor

        return ReplicateInstrumentor()



import abc
import logging
from typing import Collection

import opentelemetry.instrumentation.instrumentor
from opentelemetry.instrumentation.instrumentor import BaseInstrumentor


class NoopInstrumentor(BaseInstrumentor):
    def _uninstrument(self, **kwargs):
        return

    def instrumentation_dependencies(self) -> Collection[str]:
        return []

    def instrument(self, **kwargs):
        pass

    def uninstrument(self, **kwargs):
        pass


class Integration(abc.ABC):
    INTEGRATION_KEY: str = ""

    def __init__(self):
        self.logger = logging.getLogger("highlight_io")

    def importer(self):
        try:
            instrumentor = self.instrumentor()
            # noinspection PyProtectedMember
            conflicts = instrumentor._check_dependency_conflicts()
            if conflicts:
                raise RuntimeError(conflicts)
            return instrumentor
        except (ImportError, ModuleNotFoundError, RuntimeError) as e:
            self.logger.debug(
                "Instrumentation %s not available: %s", self.INTEGRATION_KEY, e
            )
            return NoopInstrumentor()

    def instrumentor(
        self,
    ) -> opentelemetry.instrumentation.instrumentor.BaseInstrumentor:
        raise NotImplementedError()

    def enable(self):
        self.importer().instrument()

    def disable(self):
        self.importer().uninstrument()



from highlight_io.integrations import Integration


class TransformersIntegration(Integration):
    INTEGRATION_KEY = "transformers"

    def instrumentor(self):
        from opentelemetry.instrumentation.transformers import TransformersInstrumentor

        return TransformersInstrumentor()



from highlight_io.integrations import Integration


class SQLAlchemyIntegration(Integration):
    INTEGRATION_KEY = "sqlalchemy"

    def instrumentor(self):
        from opentelemetry.instrumentation.sqlalchemy import SQLAlchemyInstrumentor

        return SQLAlchemyInstrumentor()



from highlight_io.integrations import Integration


class CohereIntegration(Integration):
    INTEGRATION_KEY = "cohere"

    def instrumentor(self):
        from opentelemetry.instrumentation.cohere import CohereInstrumentor

        return CohereInstrumentor()



from highlight_io.integrations import Integration


class LangchainIntegration(Integration):
    INTEGRATION_KEY = "langchain"

    def instrumentor(self):
        from opentelemetry.instrumentation.langchain import LangchainInstrumentor

        return LangchainInstrumentor()



from highlight_io.integrations import Integration


class QdrantInstrumentation(Integration):
    INTEGRATION_KEY = "qdrant"

    def instrumentor(self):
        from opentelemetry.instrumentation.qdrant import QdrantInstrumentor

        return QdrantInstrumentor()



from highlight_io.integrations import Integration


class WeaviateIntegration(Integration):
    INTEGRATION_KEY = "weaviate"

    def instrumentor(self):
        from opentelemetry.instrumentation.weaviate import WeaviateInstrumentor

        return WeaviateInstrumentor()



def flatten_dict(d, parent_key="", sep="."):
    items = []
    for k, v in d.items():
        new_key = parent_key + sep + k if parent_key else k
        if isinstance(v, dict):
            items.extend(flatten_dict(v, new_key, sep=sep).items())
        else:
            items.append((new_key, v))
    return dict(items)



from highlight_io.integrations import Integration


class PineconeIntegration(Integration):
    INTEGRATION_KEY = "pinecone"

    def instrumentor(self):
        from opentelemetry.instrumentation.pinecone import PineconeInstrumentor

        return PineconeInstrumentor()



from highlight_io.integrations import Integration


class Boto3SQSIntegration(Integration):
    INTEGRATION_KEY = "boto3sqs"

    def instrumentor(self):
        from opentelemetry.instrumentation.boto3sqs import Boto3SQSInstrumentor

        return Boto3SQSInstrumentor()



from highlight_io.integrations import Integration


class WatsonXIntegration(Integration):
    INTEGRATION_KEY = "watsonx"

    def instrumentor(self):
        from opentelemetry.instrumentation.watsonx import WatsonxInstrumentor

        return WatsonxInstrumentor()
